import click
from oceania.main.client import get_sequences_from_fasta_to_file


@click.group(context_settings={"help_option_names": ["-h", "--help"]})
def main():
    """A simple OceanIA command line tool."""


@main.command()
@click.argument("key", required=True, metavar="<key>")
@click.argument(
    "query_file", required=True, metavar="<query_file>", type=click.File("r")
)
@click.argument("output_format", required=True, metavar="<output_format>")
@click.argument(
    "output_file", required=True, metavar="<output_file>", type=click.File("wb")
)
def query_fasta(key, query_file, output_format, output_file):
    """Extract secuences from a fasta file in the OceanIA Storage.

    \b
    <key> object key in the OceanIA storage
    <query_file> CSV file containing the values to query.
                 Each line represents a sequence to extract in the format "sequence_id,start,end,type"
                 "sequence_id" sequence ID
                 "start" start index position of the sequence to be extracted
                 "end" end index position of the sequence to extract
                 "type" type of the sequence to extract
                        options are ["raw", "complement", "reverse_complement"]
                        type value is optional, if not provided default is "raw"
    <output_format> results format
                    options are ["csv", "fasta"]
    <output_file> name of the file to write the results
    """

    query_list = []

    for row in query_file:
        raw_values = row.rstrip().split(",")
        if len(raw_values) >= 3:
            values = [raw_values[0], int(raw_values[1]), int(raw_values[2])]
        if len(raw_values) > 3:
            values.append(raw_values[3])
        query_list.append(values)

    if len(query_list) > 0:
        get_sequences_from_fasta_to_file(
            key, query_list, output_format, output_file
        )



import requests

from .settings import settings


def download_file_from_stream(url, out_file):
    try:
        with requests.get(url, stream=True) as r:
            # based on https://stackoverflow.com/a/16696317
            r.raise_for_status()
            settings.logger.info("Downloading results...")
            for chunk in r.iter_content(chunk_size=8192):
                out_file.write(chunk)
    except requests.exceptions.HTTPError:
        return False
    return True



import requests
import pandas
import urllib
import sys
import traceback

from timeit import default_timer
from .settings import settings
from .utils import download_file_from_stream
import time

timeout = time.time() + 60 * 30  # 30 minutes from now

logger = settings.logger


def download_result(key, output):
    url = f"{settings.API_URL}query/result/{key}"
    if isinstance(output, str):
        output_handler = open(output, "wb")
    else:
        output_handler = output
    while not download_file_from_stream(url, output_handler):
        time.sleep(10)
        logger.debug(f"Waiting for {key} to be available")


def load_results_to_pandas(key):
    url = f"{settings.API_URL}query/result/{key}"
    while True:
        if time.time() > timeout:
            logger.error("TIMEOUT ERROR: ")
            logger.error("Exception in user code: 02")
            logger.error(url)
            break
        try:
            return pandas.read_csv(url)
        except urllib.error.HTTPError:
            pass
        time.sleep(10)
        logger.debug(f"Waiting for {key} to be available")


def get_sequences_from_fasta(storage_key, positions):
    start_timer = default_timer()
    url = f"{settings.API_URL}query/fasta-sequences/"
    logger.debug(f"Requesting sequences in {url}")
    logger.info("Sending request for fasta sequences")
    body = {
        "storage_key": storage_key,
        "positions": positions,
        "output_format": "csv",
    }

    try:
        response = requests.post(url, json=body)
    except ConnectionError as e:
        logger.error("CONNECTION ERROR: ")
        logger.error("Exception in user code: 01")
        logger.error(e)
        traceback.print_exc(file=sys.stdout)
        sys.exit(1)

    logger.info("Request accepted")

    results_key = response.json().get("results_key")
    logger.debug(f"Received key {results_key}")

    logger.info("Waiting for results...")

    ret = load_results_to_pandas(results_key)

    logger.debug("Done")
    end_timer = default_timer()
    logger.info(f"Done. Elapsed time: {end_timer - start_timer} seconds")

    return ret


def get_sequences_from_fasta_to_file(storage_key, positions, output_format, output):
    start_timer = default_timer()
    url = f"{settings.API_URL}query/fasta-sequences/"
    logger.debug(f"Requesting sequences in {url}")
    logger.info("Sending request for fasta sequences")
    body = {
        "storage_key": storage_key,
        "positions": positions,
        "output_format": output_format,
    }

    try:
        response = requests.post(url, json=body)
    except ConnectionError as e:
        logger.error("CONNECTION ERROR: ")
        logger.error("Exception in user code: 01")
        logger.error(e)
        traceback.print_exc(file=sys.stdout)
        sys.exit(1)

    logger.info("Request accepted")

    results_key = response.json().get("results_key")
    logger.debug(f"Received key {results_key}")

    logger.info("Waiting for results...")

    if isinstance(output, str):
        output_str = output
    else:
        output_str = output.name

    logger.debug(f"Downloading results to {output_str}")

    download_result(results_key, output)

    logger.debug("Done")
    end_timer = default_timer()
    logger.info(
        f"Done. Results saved to {output_str}. Elapsed time: {end_timer - start_timer} seconds"
    )


def list_fasta_samples():
    url = f"{settings.API_URL}query/list-queryable-samples/"
    return pandas.read_csv(url)


def list_genes_and_gaps(sample_id, page=1, page_size=50):
    url = f"{settings.API_URL}query/get-sample-genes-ids/{sample_id}?page={page}&page_size={page_size}"
    return pandas.read_csv(url, sep="\t")



import logging
import os


def setup_logger(debug):
    console_handler = logging.StreamHandler()
    formatter = logging.Formatter("[%(asctime)s] %(message)s", "%d-%m-%Y %H:%M:%S")
    console_handler.setFormatter(formatter)
    logging.basicConfig(
        level=logging.WARNING,
        handlers=[
            console_handler,
        ],
    )

    _logger = logging.getLogger(__name__)
    _logger.setLevel(logging.DEBUG if debug else logging.INFO)
    return _logger


class Settings:
    def __init__(self):
        self.QUERY_SERVICE_HOST = os.getenv(
            "OCEANIA_QUERY_SERVICE_HOST", "oceania-1.inriadev.cl"
        )
        self.QUERY_SERVICE_PORT = os.getenv("OCEANIA_QUERY_SERVICE_PORT", "7000")
        self.USE_SSL = os.getenv("OCEANIA_QUERY_SERVICE_HTTPS", "0") == "1"
        self.DEBUG = os.getenv("OCEANIA_DEBUG", "0") == "1"
        self.logger = setup_logger(self.DEBUG)

        if self.USE_SSL:
            protocol = "https://"
        else:
            protocol = "http://"
        self.API_URL = f"{protocol}{self.QUERY_SERVICE_HOST}:{self.QUERY_SERVICE_PORT}/"


settings = Settings()
